//
//  Rx5808.cpp
//  Cyclop_OSD_CPP
//
//  Created by Dominik Loher on 16.09.16.
//  Copyright © 2016 Dominik Loher. All rights reserved.
//

#include "RX5808.h"
#include <avr/io.h>
#include <util/delay.h>

// Digital pin definitions , software SPI
#define SPI_CLOCK_PIN       2
#define SLAVE_SELECT_PIN    3
#define SPI_DATA_PIN        4
#define LOW                 0
#define HIGH                1

//******************************************************************************
//* function: init SPI Pins
//******************************************************************************
RX5808::RX5808(){
    DDRD = (1<<SPI_CLOCK_PIN) | (1<<SLAVE_SELECT_PIN) | (1<<SPI_DATA_PIN);
}
//******************************************************************************
//* function: set SPI Pins
//******************************************************************************
void RX5808::setPIN( uint8_t pinNr, uint8_t state){
    if (state) {
        PORTD = 1<<pinNr ;
    }else{
        PORTD = 0<<pinNr ;
    }
}
//******************************************************************************
//* function: spi_1
//******************************************************************************
void RX5808::spi_1()
{
    
    setPIN(SPI_CLOCK_PIN, LOW);
    _delay_us(1);
    setPIN(SPI_DATA_PIN, HIGH);
    _delay_us(1);
    setPIN(SPI_CLOCK_PIN, HIGH);
    _delay_us(1);
    setPIN(SPI_CLOCK_PIN, LOW);
    _delay_us(1);
}

//******************************************************************************
//* function: spi_0
//******************************************************************************
void RX5808::spi_0()
{
    setPIN(SPI_CLOCK_PIN, LOW);
    _delay_us(1);
    setPIN(SPI_DATA_PIN, LOW);
    _delay_us(1);
    setPIN(SPI_CLOCK_PIN, HIGH);
    _delay_us(1);
    setPIN(SPI_CLOCK_PIN, LOW);
    _delay_us(1);
}

//******************************************************************************
//* function: spiEnableLow
//******************************************************************************
void RX5808::spiEnable(bool state)
{
    if (state) {
        _delay_us(1);
        setPIN(SLAVE_SELECT_PIN, HIGH);
        _delay_us(1);
        
    }else{
        _delay_us(1);
        setPIN(SLAVE_SELECT_PIN, LOW);
        _delay_us(1);
        
    }
}

//******************************************************************************
//* function: calcFrequencyData
//*         : calculates the frequency value for the syntheziser register B of
//*         : the RTC6751 circuit that is used within the RX5808/RX5880 modules.
//*         : this value is inteded to be loaded to register at adress 1 via SPI
//*         :
//*  Formula: frequency = ( N*32 + A )*2 + 479
//******************************************************************************
uint16_t RX5808::calcFrequencyData( uint16_t frequency )
{
    uint16_t N;
    uint8_t A;
    frequency = (frequency - 479) / 2;
    N = frequency / 32;
    A = frequency % 32;
    return (N << 7) |  A;
}

//******************************************************************************
//* function: setRTC6715Frequency
//*         : for a given frequency the register setting for synth register B of
//*         : the RTC6715 circuit is calculated and bitbanged via the SPI bus
//*         : please note that the synth register A is assumed to have default
//*         : values.
//*
//* SPI data: 4  bits  Register Address  LSB first
//*         : 1  bit   Read or Write     0=Read 1=Write
//*         : 13 bits  N-Register Data   LSB first
//*         : 7  bits  A-Register        LSB first
//******************************************************************************

void RX5808::setFrequency( uint16_t frequency){
    
    uint16_t sRegB;
    uint8_t i;
    
    sRegB = calcFrequencyData(frequency);
    
    // Bit bang the syntheziser register
    
    // Clock
    spiEnable(true);
    _delay_us(1);
    spiEnable(false);
    
    // Address (0x1)
    spi_1();
    spi_0();
    spi_0();
    spi_0();
    
    // Read/Write (Write)
    spi_1();
    
    // Data (16 LSB bits)
    for (i = 16; i; i--, sRegB >>= 1 ) {
        (sRegB & 0x1) ? spi_1() : spi_0();
    }
    // Data zero padding
    spi_0();
    spi_0();
    spi_0();
    spi_0();
    
    // Clock
    spiEnable(true);
    _delay_us(1);
    
    setPIN(SLAVE_SELECT_PIN, LOW);
    setPIN(SPI_CLOCK_PIN, LOW);
    setPIN(SPI_DATA_PIN, LOW);
}
