#include "max7456.h"
#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "max7456registers.h"
#include <stdlib.h>
#include "SPI.h"
//-----------------------------------------------------------------------------
// Implements Max7456::init
//---------------------------------------------------------------------------
Max7456::Max7456(){
   
    _isActivatedOsd = false;
    
    _regVm1.whole = 0b01000111;
    spi_init();
    _delay_ms(20);
    
    spi_selectSlave(true);
    spi_fast_shift(VM0_ADDRESS_WRITE);
    
    _regVm0.whole = 0x00;
    _regVm0.bits.videoSelect=1; //PAL
    _regVm0.bits.softwareResetBit = 1;
    spi_fast_shift(_regVm0.whole);
    spi_selectSlave(false);
    _delay_ms(20);
    
    spi_selectSlave(true);
    for(int x = 0 ; x < 16 ; x++)
    {
        _regRb[x].whole = 0x00;
        _regRb[x].bits.characterWhiteLevel = 2;
        spi_fast_shift(x+RB0_ADDRESS_WRITE);
        spi_fast_shift(_regRb[x].whole);
    }
    
    _regVm0.whole = 0x00;
    
    _regVm0.bits.verticalSynch = 1;
    
    spi_fast_shift(VM0_ADDRESS_WRITE);
    
    spi_fast_shift(_regVm0.whole);
    
    
    //  digitalWrite(_pinCS,HIGH);
    //
    //  digitalWrite(_pinCS,LOW);
    //SPI.transfer(VM1_ADDRESS_WRITE);
    //SPI.transfer(0x0C);
    
    spi_selectSlave(false);
    
}

//-----------------------------------------------------------------------------
// Implements Max7456::activateOSD
//-----------------------------------------------------------------------------
void Max7456::activateOSD( bool act )
{
    if(_isActivatedOsd != act)
    {
        
        _regVm0.bits.videoSelect = 1;
        if(act)
            _regVm0.bits.enableOSD = 1;
        else
            _regVm0.bits.enableOSD = 0;
        
        spi_selectSlave(true);
        spi_fast_shift(VM0_ADDRESS_WRITE);
        spi_fast_shift(_regVm0.whole);
        spi_selectSlave(false);
        _isActivatedOsd = act;
    }
}

//-----------------------------------------------------------------------------
// Implements Max7456::setDisplayOffsets
//----------------------------------------------------------------------------
void Max7456::setDisplayOffsets(byte horizontal, byte vertical)
{
    
    _regHos.whole = 0;
    _regVos.whole = 0;
    
    _regHos.bits.horizontalPositionOffset = horizontal;
    _regVos.bits.verticalPositionOffset = vertical;
    
    spi_selectSlave(true);
    spi_fast_shift(HOS_ADDRESS_WRITE);
    spi_fast_shift(_regHos.whole);
    
    spi_fast_shift(VOS_ADDRESS_WRITE);
    spi_fast_shift(_regVos.whole);
    
    spi_selectSlave(false);
}

//-----------------------------------------------------------------------------
// Implements Max7456::setBlinkParams
//-----------------------------------------------------------------------------
void Max7456::setBlinkParams(byte blinkBase, byte blinkDC)
{
    _regVm1.bits.blinkingTime = blinkBase;
    _regVm1.bits.blinkingDutyCycle = blinkDC;
    spi_selectSlave(true);
    spi_fast_shift(VM1_ADDRESS_WRITE);
    spi_fast_shift(_regVm1.whole);
    spi_selectSlave(false);
}

//-----------------------------------------------------------------------------
// Implements Max7456::print
//---------------------------------------------------------------------------
/* Map ASCII table to the MAX7456 character map.
 Note, the MAX7456 in-built character map is no where near the ascii
 table mapping and very few characters are available to map. Where
 possible we create new characters for those we need that are missing
 from the MAX7456 that we want to use and also we create some special
 characters of our own that are not ASCII chars (crosshair for example).
 These additional character definitions are listed below the table.
 Character maps we have create can be found in MAX7456CHARS.h */
uint8_t MAX7456_ascii[256] = {
    
    /* Regular ASCII table. */
    /*         00    01    02    03    04    05    06    07     08    09    0A    0B    0C    0D    0E    0F  */
    /* 00 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* 10 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* 20 */ 0x00, 0x00, 0x48, 0x00, 0x00, 0x00, 0x00, 0x46,  0x3F, 0x40, 0x00, 0x4d, 0x45, 0x49, 0x41, 0x47,
    /* 30 */ 0x0A, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,  0x08, 0x09, 0x44, 0x43, 0x4A, 0x00, 0x4B, 0x42,
    /* 40 */ 0x4C, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11,  0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19,
    /* 50 */ 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21,  0x22, 0x23, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* 60 */ 0x46, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B,  0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33,
    /* 70 */ 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B,  0x3C, 0x3D, 0x3E, 0x00, 0x00, 0x00, 0x00, 0x00,
    
    /* Extended ASCII table. */
    /*         00    01    02    03    04    05    06    07     08    09    0A    0B    0C    0D    0E    0F  */
    /* 80 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* 90 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* A0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* B0 */ 0xB0, 0x00, 0x00, 0xB3, 0xB4, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xBF,
    /* C0 */ 0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* D0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0xD9, 0xDA, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* E0 */ 0xe0, 0xe1, 0xe2, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /* F0 */ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

byte Max7456::giveMax7456CharFromAsciiChar(char ascii)
{
#ifdef MAX7456_TABLE_ASCII
        return MAX7456_ascii[ascii];
#else
    return ascii;
#endif
}

void Max7456::print(const char string[], byte x, byte y, byte blink,byte inv)
{
    char          currentChar;
    byte          size;
    byte *chars = NULL;
    
    if(!string) return;
    
    size = 0;
    currentChar = string[0];
    
    while(currentChar != '\0')
    {
        currentChar = string[++size];
    }
    
    chars = (byte*) malloc(size * sizeof(byte));
    
    for(byte i = 0 ; i < size ; i++)
    {
        chars[i] = Max7456::giveMax7456CharFromAsciiChar(string[i]);
    }
    
    printMax7456Chars(chars, size, x,  y,  blink , inv );
    free(chars);
}

void Max7456::print(double value, byte x, byte y, byte before, byte after, byte blink,byte inv)
{
    char *strValue = NULL;
    
    strValue = (char*) malloc((before+after+2)* sizeof(char));
    
    
    if(after==0)
        dtostrf(value,before+after,after,strValue);
    else
        dtostrf(value,before+after+1,after,strValue);
    
    for(int i = 0 ; i < before+after+1;i++)
    {
        if(strValue[i] == ' ' || strValue[i] == '-')
            strValue[i]='0';
    }
    if(value < 0)
        strValue[0]='-';
    if(after==0) //If the result is bigger, we truncate it so the OSD won't be falsed.
        strValue[before]='\0';
    else
        strValue[before+after+1]='\0';
    
    print(strValue,x,y,blink,inv);
    
    free(strValue);
}
//-----------------------------------------------------------------------------
// Implements Max7456::printMax7456Chars
//---------------------------------------------------------------------------

void Max7456::printMax7456Char(const byte address, byte x, byte y, byte blink, byte inv)
{
    byte ad = address;
    printMax7456Chars(&ad,1,x,y,blink,inv);
}

void Max7456::printMax7456Chars(byte chars[],byte size,byte x, byte y, byte blink ,byte inv )
{
    byte          currentCharMax7456;
    byte          posAddressLO;
    byte          posAddressHI;
    unsigned int  posAddress;
    
    posAddress = 30*y+x;
    
    posAddressHI = posAddress >> 8;
    posAddressLO = posAddress;
    
    _regDmm.whole = 0x01;
    
    _regDmm.bits.INV = inv;
    _regDmm.bits.BLK = blink;
    
    
    spi_selectSlave(true);
    spi_fast_shift(DMM_ADDRESS_WRITE);
    
    
    
    spi_fast_shift(_regDmm.whole);
    
    
    spi_fast_shift(DMAH_ADDRESS_WRITE); // set start address high
    spi_fast_shift(posAddressHI);
    
    spi_fast_shift(DMAL_ADDRESS_WRITE); // set start address low
    spi_fast_shift(posAddressLO);
    
    
    for(int i = 0; i < size ; i++)
    {
        currentCharMax7456 = chars[i];
        spi_fast_shift(DMDI_ADDRESS_WRITE);
        spi_fast_shift(currentCharMax7456);
        
    }
    
    //end character (we're done).
    spi_fast_shift(DMDI_ADDRESS_WRITE);
    spi_fast_shift(0xff);
    
    /*
     _regVm0.bits.
     SPI.transfer(VM0_ADDRESS_WRITE);
     SPI.transfer(0x4c);*/
    
    
    spi_selectSlave(false);
}

//-----------------------------------------------------------------------------
// Implements Max7456::clearScreen
//-----------------------------------------------------------------------------
void Max7456::clearScreen()
{
    _regDmm.bits.clearDisplayMemory = 1 ;
    
    spi_selectSlave(true);
    
    spi_fast_shift(DMM_ADDRESS_WRITE);
    spi_fast_shift(_regDmm.whole);
    
    //wait for operation to be complete.
    while(_regDmm.bits.clearDisplayMemory == 1 )
    {
        spi_fast_shift(DMM_ADDRESS_READ);
        _regDmm.whole = spi_fast_shift(0x00);
    }
    spi_selectSlave(false);
}
//-----------------------------------------------------------------------------
// Implements Max7456::sendCharacter
//-----------------------------------------------------------------------------
void Max7456::sendCharacter(const char chara[], uint8_t charAddress)
{
   
    activateOSD(false);
    //datasheet p38
    spi_selectSlave(true);
    spi_fast_shift(CMAH_ADDRESS_WRITE);
    spi_fast_shift(charAddress);
    
    for(byte i = 0 ; i < 54 ; i++)
    {
        spi_fast_shift(CMAL_ADDRESS_WRITE);
        spi_fast_shift(i);
        spi_fast_shift(CMDI_ADDRESS_WRITE);
        spi_fast_shift(chara[i]);
    }
    
    _regCmm = 0xA0; //To write the NVM array
    spi_fast_shift(CMM_ADDRESS_WRITE);
    spi_fast_shift(_regCmm);
    //while STAT[5] is not good, we wait.
    _regStat.bits.characterMemoryStatus = 1;
    
    while(  _regStat.bits.characterMemoryStatus ==1)
    {
        spi_selectSlave(STAT_ADDRESS_READ);
        _regStat.whole = spi_fast_shift(0x00);
    }
    
    
    spi_selectSlave(false);

}

//-----------------------------------------------------------------------------
// Implements Max7456::activateExternalVideo
//-----------------------------------------------------------------------------
void Max7456::activateExternalVideo(bool activExtVid)
{
    if(!activExtVid)
        _regVm0.bits.synchSelect = 3; //11
    else
        _regVm0.bits.synchSelect = 0; //0
    
    spi_selectSlave(true);
    spi_fast_shift(VM0_ADDRESS_WRITE);
    spi_fast_shift(_regVm0.whole);
    spi_selectSlave(false);
}
